@@ -4,11 +4,11 @@ bd.array <- function (beta, phi) {
     J <- length(beta)
     if (length(phi) != J)
         stop ("beta and phi differ in number of sessions")
-    if (sum(beta) != 1)
-        stop ("beta values do not sum to 1.0")
+    if (!all.equal(sum(beta), 1.0, tolerance = 1e-4))
+        warning ("beta values do not sum to 1.0 in bd.array")
     phi[J] <- 0
     if (any (phi<0 | phi>1))
-        stop ("phi outside range 0-1")
+        warning ("phi outside range 0-1 in bd.array")
     pbd <- matrix(NA, nrow = J, ncol = J, dimnames = list(b=1:J, d=1:J))
     for (b in 1:J) {
         for (d in b:J) {
