@@ -12,57 +12,10 @@
 ## 2020-12-12 mqsetup moved from prwisecr.R
 ## 2021-03-30 logmultinomial local version of function
 ## 2021-04-24 2.0.0 stratification
+## 2021-07-23 get.model.matrix wrapper for calls to model.matrix
+## 2021-07-31 new movement models, general tidy up
 ###############################################################################
 
-typecode <- function (type) {
-    type <- switch(type, 
-        CJS = 1, CJSmte = 5,
-        JSSAf = 4, JSSAl = 3, JSSAb = 2, JSSAg = 22, JSSAk = 28, 
-        JSSAfCL = 15, JSSAlCL = 16, JSSAbCL = 17, JSSAgCL = 23, JSSAkCL = 29,
-        JSSAB = 18, JSSAN = 19,
-        
-        CJSsecr = 6, 
-        JSSAsecrfCL = 9, JSSAsecrlCL = 10, JSSAsecrbCL = 11, JSSAsecrgCL = 25, 
-        JSSAsecrf = 7, JSSAsecrl = 12, JSSAsecrb = 13, JSSAsecrg = 24, 
-        JSSAsecrB = 14, JSSAsecrD = 8,
-        
-        Pradel = 20, JSSARET = 21, Pradelg = 26, JSSAfgCL = 27,
-        secrCL = 30, secrD = 31, 
-        
-        # added aliases 2020-11-01
-        PLBf = 15, PLBl = 16, PLBb = 17, PLBg = 23, PLBk = 29,
-        PLBsecrf = 9, PLBsecrl = 10, PLBsecrb = 11, PLBsecrg = 25, 
-        
-        -1)
-    type
-}
-
-movecode <- function (movementmodel) {
-    switch (movementmodel, 
-        static = 0, uncorrelated = 1, normal = 2, 
-        exponential = 3, user = 4, t2D = 5, uniform = 6, annular = 7, 
-        annular2 = 8, annularR = 9, 
-        BVN = 2, BVE = 3, BVT = 5        ## added 2021-05-31
-    )
-}
-
-sparsebool <- function(sparsekernel, movementmodel) {
-  out <- sparsekernel
-  if (is.null(out)) {
-      # for short-term backward compatibility
-      out <- movementmodel %in% c('normalS', 'exponentialS', 't2DS')  
-  }
-  out
-}
-  
-edgemethodcode <- function (edgemethod) {
-    if (is.null(edgemethod)) edgemethod <- 'none' 
-    if (!edgemethod %in% c('none','wrap','truncate')) {
-        stop ("unrecognised edge method - should be 'none','wrap' or 'truncate'")
-    }
-    switch (edgemethod, none = 0, wrap = 1, truncate = 2, 0)
-}
-
 .openCRstuff <- new.env()
 #.openCRstuff$packageType <- ' pre-release'
 .openCRstuff$packageType <- ''
@@ -90,6 +43,48 @@ edgemethodcode <- function (edgemethod) {
 .openCRstuff$traplearnedresponses <- c('bk', 'Bk', 'k', 'ksession', 'Ksession',
     'bksession', 'Bksession')
 
+# recognised movement models, excluding static and uncorrelated
+.openCRstuff$movementmodels <- c(
+  'normal','BVN',
+  'exponential','BVE',
+  't2D','BVT',
+  'uniform',
+  'annular', 
+  'frE', 
+  'frG', 
+  'frL', 
+  'BVNzi',
+  'BVEzi',
+  'uniformzi',
+  'frEzi',
+  'uncorrelatedzi')
+
+################################################################################
+
+typecode <- function (type) {
+  type <- switch(type, 
+    CJS = 1, CJSmte = 5,
+    JSSAf = 4, JSSAl = 3, JSSAb = 2, JSSAg = 22, JSSAk = 28, 
+    JSSAfCL = 15, JSSAlCL = 16, JSSAbCL = 17, JSSAgCL = 23, JSSAkCL = 29,
+    JSSAB = 18, JSSAN = 19,
+    
+    CJSsecr = 6, 
+    JSSAsecrfCL = 9, JSSAsecrlCL = 10, JSSAsecrbCL = 11, JSSAsecrgCL = 25, 
+    JSSAsecrf = 7, JSSAsecrl = 12, JSSAsecrb = 13, JSSAsecrg = 24, 
+    JSSAsecrB = 14, JSSAsecrD = 8,
+    
+    Pradel = 20, JSSARET = 21, Pradelg = 26, JSSAfgCL = 27,
+    secrCL = 30, secrD = 31, 
+    
+    # added aliases 2020-11-01
+    PLBf = 15, PLBl = 16, PLBb = 17, PLBg = 23, PLBk = 29,
+    PLBsecrf = 9, PLBsecrl = 10, PLBsecrb = 11, PLBsecrg = 25, 
+    
+    -1)
+  type
+}
+################################################################################
+
 # detection function numbers 14-20 are subset of those used by 'secr'
 detectionfunctionnumber <- function (detname) {
     dfn <- match (toupper(detname), .openCRstuff$DFN)
@@ -99,8 +94,79 @@ detectionfunctionnumber <- function (detname) {
         stop ("detection function ", detname, " not recognised in openCR")
     dfn+13
 }
+################################################################################
 
-replacedefaults <- function (default, user) replace(default, names(user), user)
+movecode <- function (movementmodel) {
+  switch (movementmodel, 
+    static        = 0, 
+    uncorrelated  = 1, 
+    normal        = 2, 
+    BVN           = 2, 
+    exponential   = 3, 
+    BVE           = 3, 
+    user          = 4, 
+    t2D           = 5, 
+    BVT           = 5,
+    uniform       = 6, 
+    annular       = 7, 
+    annular2      = 8, 
+    annularR      = 9, 
+    frE           = 10, 
+    frG           = 11, 
+    frL           = 12, 
+    BVNzi          = 13,
+    BVEzi          = 14,
+    uniformzi      = 15,
+    frEzi          = 16,
+    uncorrelatedzi = 17
+  )
+}
+################################################################################
+
+nparmove <- function (movementmodel) {
+  switch(movementmodel, 
+    static        = 0, 
+    uncorrelated  = 0, 
+    normal        = 1, 
+    BVN           = 1, 
+    exponential   = 1, 
+    BVE           = 1, 
+    t2D           = 2, 
+    BVT           = 2, 
+    uniform       = 0, 
+    annular       = 1, 
+    annular2      = 2, 
+    annularR      = 2, 
+    frE           = 1, 
+    frG           = 2, 
+    frL           = 2,
+    BVNzi          = 2,
+    BVEzi          = 2,
+    uniformzi      = 1,
+    frEzi          = 2,
+    uncorrelatedzi = 1,  
+    0)
+}
+################################################################################
+
+edgemethodcode <- function (edgemethod) {
+  if (is.null(edgemethod)) edgemethod <- 'none' 
+  if (!edgemethod %in% c('none','wrap','truncate')) {
+    stop ("unrecognised edge method - should be 'none','wrap' or 'truncate'")
+  }
+  switch (edgemethod, 
+    none     = 0, 
+    wrap     = 1,
+    truncate = 2, 
+    0)
+}
+################################################################################
+
+replacedefaults <- function (default, user) {
+  replace(default, names(user), user)
+}
+
+################################################################################
 
 discreteN <- function (n, N) {
     tN <- trunc(N)
@@ -108,12 +174,14 @@ discreteN <- function (n, N) {
         replace = T, size = n)
     else rep(tN,n)
 }
+################################################################################
 
 memo <- function (text, trace) {
     ## could use message(text), but does not immediately flush console
     if (trace) { cat (text, '\n')
     flush.console() }
 }
+################################################################################
 
 pad1 <- function (x, n) {
 ## pad x to length n with dummy (first value)
@@ -125,6 +193,7 @@ pad1 <- function (x, n) {
     else out <- c(x, rep(x[1], n-length(x)))
     out
 }
+################################################################################
 
 padarray <- function (x, dims) {
     temp <- array(dim=dims)
@@ -205,13 +274,13 @@ add.cl <- function (df, alpha, loginterval, lowerbound = 0) {
     }
     df
 }
-
 ###############################################################################
 
 model.string <- function (model, userDfn) {
     temp <- paste (names(model), as.character(model), collapse=' ', sep='')
     temp
 }
+###############################################################################
 
 fixed.string <- function (fixed) {
     # copied from 'secr'
@@ -220,7 +289,6 @@ fixed.string <- function (fixed) {
 }
 ############################################################################################
 
-
 ## lifted from secr score.test
 
 mapbeta <- function (parindx0, parindx1, beta0, betaindex)
@@ -283,11 +351,13 @@ primarysessions <- function(intervals) {
     primarysession <- cumsum(c(0,intervals))
     match(primarysession, unique(primarysession))
 }
+###############################################################################
 
 secondarysessions <- function(intervals) {
     primary <- primarysessions(intervals)
     unname(unlist(sapply(table(primary), seq_len)))  
 }
+###############################################################################
 
 primaryonly <- function(object) {
     intervals <- intervals(object)
@@ -315,12 +385,19 @@ primaryonly <- function(object) {
     }
     object
 }
+###############################################################################
 
-# expandfreq <- function (CH, freq) {
-#     f <- rep(1:nrow(CH), freq)    
-#     out <- CH[f,,]
-#     out
-# }
+primaryintervals <- function (object, ...) {
+  ## 2021-04-26 list, one vector per stratum
+  ## for backward compatibility -
+  out <- list(object$intervals)
+  if (is.null(out[[1]])) {
+    out <- object$primaryintervals 
+  }
+  if (!is.list(out)) stop ("primarysessions should be stratum list")
+  out
+}
+###############################################################################
 
 ##### constrain to 0 < sum(p) <= 1.0
 invmlogit <- function (x, fillin = TRUE) {
@@ -337,7 +414,7 @@ transform <- function (x, link) {
     switch (link,
         identity = x,
         log = log(x),
-        log05 = log(x-0.5),
+        log1 = log(x-1),
         neglog = log(-x),
         logit = logit(x),
         loglog = -log(-log(x)),
@@ -353,7 +430,7 @@ untransform <- function (beta, link) {
     switch (link,
         identity = beta,
         log = exp(beta),
-        log05 = exp(beta) + 0.5,
+        log1 = exp(beta) + 1,
         neglog = -exp(beta),
         logit = invlogit(beta),
         loglog = exp(-exp(-beta)),
@@ -367,7 +444,7 @@ se.untransform <- function (beta, sebeta, link) {
     switch (link,
         identity = sebeta,
         log = exp(beta) * sqrt(exp(sebeta^2)-1),
-        log05 =  exp(beta) * sqrt(exp(sebeta^2)-1),
+        log1 =  exp(beta) * sqrt(exp(sebeta^2)-1),
         neglog = exp(beta) * sqrt(exp(sebeta^2)-1),
         logit = invlogit(beta) * (1-invlogit(beta)) * sebeta,
         loglog = -exp(-exp(-beta)) * sebeta * -exp(-beta),
@@ -384,7 +461,7 @@ Xtransform <- function (real, linkfn, varnames) {
         out[i] <- switch (linkfn[[vn]],
             identity = real[i],
             log = log(real[i]),
-            log05 = log(real[i]-0.5),
+            log1 = log(real[i]-1),
             neglog = log(-real[i]),
             logit = logit(real[i]),
             loglog = -log(-log(real[i])),
@@ -403,7 +480,7 @@ se.Xtransform <- function (real, sereal, linkfn, varnames) {
         out[i] <- switch (linkfn[[vn]],
             identity = sereal[i],
             log = log((sereal[i]/real[i])^2 + 1)^0.5,
-            log05 = log((sereal[i]/real[i])^2 + 1)^0.5,
+            log1 = log((sereal[i]/real[i])^2 + 1)^0.5,
             neglog = log((sereal[i]/-real[i])^2 + 1)^0.5,
             logit = sereal[i] / real[i] / (1 - real[i]),
             loglog = -real[i] * sereal[i] * log(real[i]),
@@ -422,7 +499,7 @@ Xuntransform <- function (beta, linkfn, varnames) {
         out[i] <- switch (linkfn[[vn]],
             identity = beta[i],
             log = exp(beta[i]),
-            log05 = exp(beta[i]) + 0.5,
+            log1 = exp(beta[i]) + 1,
             neglog = -exp(beta[i]),
             logit = invlogit(beta[i]),
             loglog = exp(-exp(-beta[i])),
@@ -447,8 +524,8 @@ se.Xuntransform <- function (beta, sebeta, linkfn, varnames)
         vn <- varnames[i]
         out[i] <- switch (linkfn[[vn]],
             identity = sebeta[i],
-            log = exp(beta[i]) * sqrt(exp(sebeta[i]^2)-1),
-            log05 = exp(beta[i]) * sqrt(exp(sebeta[i]^2)-1),
+            log    = exp(beta[i]) * sqrt(exp(sebeta[i]^2)-1),
+            log1   = exp(beta[i]) * sqrt(exp(sebeta[i]^2)-1),
             neglog = exp(beta[i]) * sqrt(exp(sebeta[i]^2)-1),
             logit = invlogit(beta[i]) * (1-invlogit(beta[i])) * sebeta[i],
             loglog = NA, 
@@ -555,6 +632,23 @@ getvalidlevels <- function (type, parnames, J, CJSp1) {
     validlevels
 }
 ############################################################################################
+
+# 2021-07-23
+# wrapper for call to model.matrix that is used in openCR.design, lpredictor etc.
+
+get.model.matrix <- function(formula, field, dframe, validlevels, contrasts, ...) {
+  # adjust for unidentifiable parameters
+  dframe <- adjustlevels(field, dframe, validlevels)
+  # avoid contrasts error 2021-04-26
+  if (length(levels(dframe$stratum))==1) {
+    dframe$stratum <- rep(1, nrow(dframe))
+  }
+  # drop contrasts not relevant to this formula
+  localcontrasts <- contrasts[names(contrasts) %in% all.vars(formula)]
+  if (length(localcontrasts)==0) localcontrasts <- NULL
+  model.matrix(formula, data = dframe, contrasts.arg = localcontrasts, ...)
+}
+
 lpredictor <- function (model, newdata, indx, beta, field, beta.vcv=NULL, 
   validlevels, contrasts = NULL) {
     vars <- all.vars(model)
@@ -587,13 +681,16 @@ lpredictor <- function (model, newdata, indx, beta, field, beta.vcv=NULL,
         notOK <- !validlevels[field,newdata$t]   ## BEFORE adjust levels
     }
     
-    newdata1 <- adjustlevels(field, newdata, validlevels)
+    # newdata1 <- adjustlevels(field, newdata, validlevels)
     # avoid contrasts error 2021-04-26
-    if (length(levels(newdata1$stratum))==1) {
-      newdata1$stratum <- rep(1, nrow(newdata1))
-    }
-    mat <- model.matrix(model, data=newdata1, contrasts.arg = contrasts)
+    # if (length(levels(newdata1$stratum))==1) {
+    #   newdata1$stratum <- rep(1, nrow(newdata1))
+    # }
+    # mat <- model.matrix(model, data=newdata1, contrasts.arg = contrasts)
 
+    # new wrapper function 2021-07-23
+    mat <- get.model.matrix (model, field, newdata, validlevels, contrasts)
+      
     ## drop pmix beta0 column from design matrix (always zero)
     if (field=='pmix') {
         mat <- mat[,-1,drop=FALSE]
@@ -739,7 +836,6 @@ getcumss <- function(capthist) {
 }
 ############################################################################################
 
-############################################################################################
 fillpmix2 <- function (nc, nmix, PIA, realparval) {
     pmix <- matrix(1, nrow = nc, ncol = nmix) 
     if (nmix>1) {        
@@ -750,7 +846,7 @@ fillpmix2 <- function (nc, nmix, PIA, realparval) {
 }
 ############################################################################################
 
-age.matrix <- function (capthist, initialage = 0, minimumage = 0, maximumage = 1, collapse = FALSE) #, bysession = FALSE) 
+age.matrix <- function (capthist, initialage = 0, minimumage = 0, maximumage = 1, collapse = FALSE) 
 {    
     makeage <- function (n) {
         ch <- capthist[n,,,drop=FALSE]
@@ -862,7 +958,6 @@ unsqueeze <- function(x) {
 }
 ############################################################################################
 
-
 ## code used in openCR.fit to put capthist object in a standard form
 ## 2018-02-11
 ## modified 2021-04-18 for stratified input
@@ -951,9 +1046,12 @@ complete.beta.vcv <- function (object) {
     if (!is.null(fb)) {
         names(fb) <- unlist(sapply(object$design$designMatrices, colnames))
         nbeta <- length(fb)
-        beta.vcv <- matrix(0, nrow = nbeta, ncol = nbeta, dimnames=list(names(fb), names(fb)))
-        if (!is.null(object$beta.vcv))
-            beta.vcv[is.na(fb[row(beta.vcv)]) & is.na(fb[col(beta.vcv)])] <- object$beta.vcv
+        beta.vcv <- matrix(0, nrow = nbeta, ncol = nbeta, 
+          dimnames = list(names(fb), names(fb)))
+        if (!is.null(object$beta.vcv)) {
+          beta.vcv[is.na(fb[row(beta.vcv)]) & is.na(fb[col(beta.vcv)])] <- 
+            object$beta.vcv
+        }
     }
     else {
         beta.vcv <- object$beta.vcv
@@ -1010,6 +1108,7 @@ rectwrap <- function(oldx, oldy, newx, newy, concat = TRUE) {
   else 
     list(newx=newx, newy=newy)
 }
+###############################################################################
 
 mqsetup <- function (
   mask,      ## x,y points on mask (first x, then y)
@@ -1042,7 +1141,6 @@ mqsetup <- function (
     matrix(i, nrow = nrow(mask), ncol = nrow(kernel))
   }
 }
-
 ###############################################################################
 
 ## local logmultinomial 2021-03-30, 2021-04-19
@@ -1060,7 +1158,7 @@ logmultinom <- function (capthist, stratified = FALSE) {
   }
 }
 
-##############################################
+###############################################################################
 xydist <- function (xy1, xy2) {
     nr <- nrow(xy1)
     nc <- nrow(xy2)
@@ -1070,8 +1168,8 @@ xydist <- function (xy1, xy2) {
     y2 <- matrix(xy2[,2], nr, nc, byrow=T)
     max(abs(x1-x2), abs(y1-y2))
 }
+###############################################################################
 
-##############################################
 getdistmat <- function (traps, mask, HPX = FALSE) {
   ## Static distance matrix
   if (HPX) {
@@ -1099,20 +1197,7 @@ getdistmat <- function (traps, mask, HPX = FALSE) {
     }
   }
 }
-
-###################################################
-
-primaryintervals <- function (object, ...) {
-  ## 2021-04-26 list, one vector per stratum
-  ## for backward compatibility -
-  out <- list(object$intervals)
-  if (is.null(out[[1]])) {
-    out <- object$primaryintervals 
-  }
-  if (!is.list(out)) stop ("primarysessions should be stratum list")
-  out
-}
-#--------------------------------------------------------------------------
+###############################################################################
 
 # function to ensure covariates are in standard form (stratum list of dataframes)
 stdcovlist <- function (cov, covname, nstrata, expected = NULL) {
@@ -1162,7 +1247,7 @@ stdcovlist <- function (cov, covname, nstrata, expected = NULL) {
       dflist
   }
 }
-#--------------------------------------------------------------------------
+###############################################################################
 
 get.nmix <- function (model) {
     # simplified local version of 'secr' function get.nmix
@@ -1179,5 +1264,5 @@ get.nmix <- function (model) {
     }
     nmix
 }
-############################################################################################
+###############################################################################
 
