@@ -193,6 +193,10 @@ fitd <- openCR.fit(ovenCH, model = p ~ -1+t,
 coef(fit0)
 coef(fitd)
 
+## ----dummy2, eval = TRUE, cache = mycache-----------------------------------------------
+fitd2 <- openCR.fit(ovenCH, model = p~t, details = list(dummyvariablecoding = 't'))
+coef(fitd2)
+
 ## ----contrsum, eval = TRUE, cache = mycache---------------------------------------------
 fit <- openCR.fit(dipperCH, model = phi~t, details = list(contrasts = list(t = contr.sum)))
 invlogit(coef(fit)['phi',c('beta','lcl','ucl')])
